@@ -29,19 +29,13 @@ define([
       var normalized_list = list.map(function(_item) {
         return _item.toLowerCase();
       });
-      return normalized_list.indexOf(item.toLowerCase()) != -1;
+      return normalized_list.indexOf(item.toLowerCase()) !== -1;
     };
 
     var includes_extension = function(filepath, extensionslist) {
       return item_in(extension(filepath), extensionslist);
     };
 
-    var json_or_xml_container_mimetype = function(mimetype) {
-      // Match */*+json or */*+xml
-      return (mimetype.substring(mimetype.length - 5) == '+json'
-              || mimetype.substring(mimetype.length - 4) == '+xml');
-    };
-
     function name_sorter(ascending) {
         return (function(a, b) {
             if (type_order[a['type']] < type_order[b['type']]) {
@@ -58,7 +52,7 @@ define([
             }
             return 0;
         });
-    };
+    }
 
     function modified_sorter(ascending) {
         var order = ascending ? 1 : 0;
@@ -68,9 +62,34 @@ define([
         });
     }
 
+    function size_sorter(ascending) {
+        var order = ascending ? 1 : 0;
+        // directories have file size of undefined
+        return (function(a, b) {
+          if (a.size === undefined) {
+             return (ascending) ? -1 : 1;
+          }
+
+          if (b.size === undefined) {
+             return (ascending) ? 1 : -1;
+          }
+
+          if (a.size > b.size) {
+            return (ascending) ? -1 : 1;
+          }
+
+          if (b.size > a.size) {
+            return (ascending) ? 1 : -1;
+          }
+
+          return 0;
+        });
+    }
+
     var sort_functions = {
         'sort-name': name_sorter,
-        'last-modified': modified_sorter
+        'last-modified': modified_sorter,
+        'file-size': size_sorter
     };
 
     var NotebookList = function (selector, options) {
@@ -102,6 +121,7 @@ define([
         this.sessions = {};
         this.base_url = options.base_url || utils.get_body_data("baseUrl");
         this.notebook_path = options.notebook_path || utils.get_body_data("notebookPath");
+        this.initial_notebook_path = this.notebook_path;
         this.contents = options.contents;
         if (this.session_list && this.session_list.events) {
             this.session_list.events.on('sessions_loaded.Dashboard',
@@ -148,11 +168,10 @@ define([
             $('#new-file').click(function(e) {
                 var w = window.open('', IPython._target);
                 that.contents.new_untitled(that.notebook_path || '', {type: 'file', ext: '.txt'}).then(function(data) {
-                    var url = utils.url_path_join(
+                    w.location = utils.url_path_join(
                         that.base_url, 'edit',
                         utils.encode_uri_components(data.path)
                     );
-                    w.location = url;
                 }).catch(function (e) {
                     w.close();
                     dialog.modal({
@@ -169,6 +188,7 @@ define([
                     console.warn('Error during New file creation', e);
                 });
                 that.load_sessions();
+                e.preventDefault();
             });
             $('#new-folder').click(function(e) {
                 that.contents.new_untitled(that.notebook_path || '', {type: 'directory'})
@@ -189,6 +209,7 @@ define([
                     console.warn('Error during New directory creation', e);
                 });
                 that.load_sessions();
+                e.preventDefault();
             });
 
             // Bind events for action buttons.
@@ -213,6 +234,7 @@ define([
                     that.select('select-all');
                 }
             });
+
             $('#button-select-all').click(function (e) {
                 // toggle checkbox if the click doesn't come from the checkbox already
                 if (!$(e.target).is('input[type=checkbox]')) {
@@ -228,7 +250,7 @@ define([
                 var sort_on = e.target.id;
 
                 // Clear sort indications in UI
-                $(".sort-action i").removeClass("fa-arrow-up").removeClass("fa-arrow-down")
+                $(".sort-action i").removeClass("fa-arrow-up").removeClass("fa-arrow-down");
 
                 if ((that.sort_id === sort_on) && (that.sort_direction === 1)) {
                     that.sort_list(sort_on, 0);
@@ -258,8 +280,7 @@ define([
         var files;
         if(dropOrForm === 'drop'){
             files = event.originalEvent.dataTransfer.files;
-        } else
-        {
+        } else  {
             files = event.originalEvent.target.files;
         }
 
@@ -279,12 +300,11 @@ define([
             });
         };
 
-        for (var i = 0; i < files.length; i++) {
-            var f = files[i];
+        Array.from(files).forEach(function(f) {
             var name_and_ext = utils.splitext(f.name);
             var file_ext = name_and_ext[1];
 
-            if (f.size > this._max_upload_size_mb * 1024 * 1024) {
+            if (f.size > that._max_upload_size_mb * 1024 * 1024) {
             	var body_msg = i18n.msg.sprintf(i18n.msg._("The file size is %d MB. Do you still want to upload it?"),
             			Math.round(f.size / (1024 * 1024)));
                 dialog.modal({
@@ -318,12 +338,12 @@ define([
                 reader.onload = reader_onload;
                 reader.onerror = reader_onerror;
             }
-        }
-        // Replace the file input form wth a clone of itself. This is required to
+        });
+        // Clear fileinput value. This is required to
         // reset the form. Otherwise, if you upload a file, delete it and try to
         // upload it again, the changed event won't fire.
         var form = $('input.fileinput');
-        form.replaceWith(form.clone(true));
+        form.val('');
         return false;
     };
 
@@ -354,6 +374,69 @@ define([
 
     NotebookList.prototype.load_list = function () {
         var that = this;
+        // Add an event handler browser back and forward events
+        window.onpopstate = function(e) {
+            var path = (window.history.state && window.history.state.path) ?
+                    window.history.state.path : that.initial_notebook_path;
+            that.update_location(path);
+        };
+        var breadcrumb = $('.breadcrumb');
+        breadcrumb.empty();
+        var list_item = $('<li/>');
+        var root_url = utils.url_path_join(that.base_url, '/tree');
+        var root = $('<li/>').append(
+            $("<a/>")
+            .attr('href', root_url)
+            .attr('title', i18n.msg._('Link to root folder'))
+            .append(
+                $("<i/>")
+                .addClass('fa fa-folder')
+            )
+            .click(function(e) {
+                // Allow the default browser action when the user holds a modifier (e.g., Ctrl-Click)
+                if(e.altKey || e.metaKey || e.shiftKey) {
+                    return true;
+                }
+                var path = '';
+                window.history.pushState(
+                    {path: path},
+                    'Home',
+                    utils.url_path_join(that.base_url, 'tree')
+                );
+                that.update_location(path);
+                return false;
+            })
+        );
+        breadcrumb.append(root);
+        var path_parts = [];
+        this.notebook_path.split('/').forEach(function(path_part) {
+            path_parts.push(path_part);
+            var path = path_parts.join('/');
+            var url = utils.url_path_join(
+                that.base_url,
+                '/tree',
+                utils.encode_uri_components(path)
+            );
+            var crumb = $('<li/>').append(
+                $('<a/>')
+                .attr('href', url)
+                .text(path_part)
+                .click(function(e) {
+                    // Allow the default browser action when the user holds a modifier (e.g., Ctrl-Click)
+                    if(e.altKey || e.metaKey || e.shiftKey) {
+                        return true;
+                    }
+                    window.history.pushState(
+                        {path: path},
+                        path,
+                        url
+                    );
+                    that.update_location(path);
+                    return false;
+                })
+            );
+            breadcrumb.append(crumb);
+        });
         this.contents.list_contents(that.notebook_path).then(
             $.proxy(this.draw_notebook_list, this),
             function(error) {
@@ -362,6 +445,17 @@ define([
         );
     };
 
+    NotebookList.prototype.update_location = function (path) {
+        this.notebook_path = path;
+        $('body').attr('data-notebook-path', path);
+        // Update the file tree list without reloading the page
+        this.load_list();
+        // Update the page title so the browser tab reflects it
+        // Match how the title appears with a trailing slash or
+        // "Home" if the page loads from the server.
+        $('title').text(path ? path+'/' : i18n.msg._("Home"));
+    };
+
     /**
      * Draw the list of notebooks
      * @method draw_notebook_list
@@ -401,7 +495,7 @@ define([
             model = {
                 type: 'directory',
                 name: '..',
-                path: utils.url_path_split(path)[0],
+                path: utils.url_path_split(path)[0]
             };
             this.add_link(model, item);
             offset += 1;
@@ -450,7 +544,7 @@ define([
         var item = $("<div/>")
             .addClass("col-md-12")
             .appendTo(row);
-        
+
         var checkbox;
         if (selectable !== undefined) {
             checkbox = $('<input/>')
@@ -471,10 +565,27 @@ define([
             .addClass("item_name")
             .appendTo(link);
 
+        var div = $('<div/>')
+            .addClass('pull-right')
+            .appendTo(item);
+
+        var buttons = $('<div/>')
+            .addClass("item_buttons pull-left")
+            .appendTo(div);
+
+        var div2 = $('<div/>')
+            .addClass('pull-right')
+            .appendTo(div);
+
         $("<span/>")
             .addClass("item_modified")
+            .addClass("pull-left")
+            .appendTo(div2);
+
+        $("<span/>")
+            .addClass("file_size")
             .addClass("pull-right")
-            .appendTo(item);
+            .appendTo(div2);
 
         if (selectable === false) {
             checkbox.css('visibility', 'hidden');
@@ -489,10 +600,6 @@ define([
             });
         }
 
-        var buttons = $('<div/>')
-            .addClass("item_buttons  pull-right")
-            .appendTo(item);
-
         $('<div/>')
             .addClass('running-indicator')
             .text(i18n.msg._('Running'))
@@ -511,13 +618,13 @@ define([
     NotebookList.icons = {
         directory: 'folder_icon',
         notebook: 'notebook_icon',
-        file: 'file_icon',
+        file: 'file_icon'
     };
 
     NotebookList.uri_prefixes = {
         directory: 'tree',
         notebook: 'notebooks',
-        file: 'edit',
+        file: 'edit'
     };
 
     /**
@@ -544,7 +651,7 @@ define([
       var ipynb_extensions = ['ipynb'];
       return includes_extension(model.path, ipynb_extensions);
     };
-    
+
     NotebookList.prototype._is_editable = function(model) {
       // Allow any file to be "edited"
       // Non-text files will display the following error:
@@ -553,17 +660,23 @@ define([
       //   See Console for more details.
       return true;
     };
-    
+
     NotebookList.prototype._is_viewable = function(model) {
       var html_types = ['htm', 'html', 'xhtml', 'xml', 'mht', 'mhtml'];
       var media_extension = ['3gp', 'avi', 'mov', 'mp4', 'm4v', 'm4a', 'mp3', 'mkv', 'ogv', 'ogm', 'ogg', 'oga', 'webm', 'wav'];
       var image_type = ['bmp', 'gif', 'jpg', 'jpeg', 'png', 'webp'];
       var other_type = ['ico'];
       var viewable_extensions = [].concat(html_types, media_extension, image_type, other_type);
-      return model.mimetype === 'text/html' 
+      return model.mimetype === 'text/html'
         || includes_extension(model.path, viewable_extensions);
     };
 
+    // Files like PDF that should be opened using `/files` prefix
+    NotebookList.prototype._is_pdflike = function(model) {
+      var pdflike_extensions = ['pdf'];
+      return includes_extension(model.path, pdflike_extensions);
+    };
+
     /**
      * Handles when any row selector checkbox is toggled.
      */
@@ -576,7 +689,6 @@ define([
         var has_running_notebook = false;
         var has_directory = false;
         var has_file = false;
-        var that = this;
         var checked = 0;
         $('.list_item :checked').each(function(index, item) {
             var parent = $(item).parent().parent();
@@ -708,6 +820,23 @@ define([
         checked = bidi.applyBidi(checked);
         $('#counter-select-all').html(checked===0 ? '&nbsp;' : checked);
 
+        //#issue 3961, update the checkbox aria-label when it changed
+        if(selected.length>=1){
+          if($('#select-all').prop("checked")){
+            // $('#button-select-all').attr("aria-label", i18n.msg._("Selected All "+ selected.length +" items"));
+            var msg1 = i18n.msg._("Selected All %d items")
+            $('#button-select-all').attr("aria-label", i18n.msg.sprintf(msg1, selected.length));
+          }
+          else{
+            // $('#button-select-all').attr("aria-label", i18n.msg._("Selected, "+ selected.length+" items"));
+            var msg2 = i18n.msg._("Selected, %d items")
+            $('#button-select-all').attr("aria-label", i18n.msg.sprintf(msg2, selected.length));
+          }
+        }
+        else{
+          $('#button-select-all').attr("aria-label", i18n.msg._("Select All/None"));
+        }
+
         // If at aleast on item is selected, hide the selection instructions.
         if (checked > 0) {
             $('.dynamic-instructions').hide();
@@ -717,6 +846,7 @@ define([
     };
 
     NotebookList.prototype.add_link = function (model, item) {
+        var that = this;
         var running = (model.type === 'notebook' && this.sessions[model.path] !== undefined);
         item.data('name',model.name);
         item.data('path', model.path);
@@ -732,6 +862,10 @@ define([
         {
             uri_prefix = 'view';
         }
+        if (model.type === 'file' && this._is_pdflike(model))
+        {
+            uri_prefix = 'files';
+        }
         if (model.type === 'file' && this._is_notebook(model))
         {
             uri_prefix = 'notebooks';
@@ -752,12 +886,33 @@ define([
         // directory nav doesn't open new tabs
         // files, notebooks do
         if (model.type !== "directory") {
-            link.attr('target',IPython._target);
+            link.attr('target', IPython._target);
+        } else {
+            // Replace with a click handler that will use the History API to
+            // push a new route without reloading the page if the click is
+            // not modified (e.g., Ctrl-Click)
+            link.click(function (e) {
+                if(e.altKey || e.metaKey || e.shiftKey) {
+                    return true;
+                }
+                window.history.pushState({
+                    path: model.path
+                }, model.path, utils.url_path_join(
+                    that.base_url,
+                    'tree',
+                    utils.encode_uri_components(model.path)
+                ));
+                that.update_location(model.path);
+                return false;
+            });
         }
-        
+
         // Add in the date that the file was last modified
         item.find(".item_modified").text(utils.format_datetime(model.last_modified));
         item.find(".item_modified").attr("title", moment(model.last_modified).format("YYYY-MM-DD HH:mm"));
+
+        var filesize = utils.format_filesize(model.size);
+        item.find(".file_size").text(filesize || '\xA0');
     };
 
 
@@ -804,7 +959,7 @@ define([
             success : function () {
                 that.load_sessions();
             },
-            error : utils.log_ajax_error,
+            error : utils.log_ajax_error
         };
 
         var session = this.sessions[path];
@@ -827,7 +982,11 @@ define([
         var item_path = this.selected[0].path;
         var item_name = this.selected[0].name;
         var item_type = this.selected[0].type;
-        var input = $('<input/>').attr('type','text').attr('size','25').addClass('form-control')
+        var input = $('<input/>')
+            .attr('type','text')
+            .attr('size','25')
+            .attr('aria-labelledby','rename-message')
+            .addClass('form-control')
             .val(item_name);
         var rename_msg = function (type) {
         	switch(type) {
@@ -836,7 +995,7 @@ define([
         	case 'notebook': return i18n.msg._("Enter a new notebook name:");
         	default: return i18n.msg._("Enter a new name:");
         	}
-        }
+        };
         var rename_title = function (type) {
            	switch(type) {
            	case 'file': return i18n.msg._("Rename file");
@@ -844,14 +1003,15 @@ define([
            	case 'notebook': return i18n.msg._("Rename notebook");
            	default: return i18n.msg._("Rename");
            	}
-        }
+        };
         var dialog_body = $('<div/>').append(
             $("<p/>").addClass("rename-message")
+                .attr('id', 'rename-message')
                 .text(rename_msg(item_type))
         ).append(
             $("<br/>")
         ).append(input);
-        
+
         // This statement is used simply so that message extraction
         // will pick up the strings.  The actual setting of the text
         // for the button is in dialog.js.
@@ -919,10 +1079,10 @@ define([
         }
 
         // Open a dialog to enter the new path, with current path as default.
-        var input = $('<input/>').attr('type','text').attr('size','25').addClass('form-control')
+        var input = $('<input/>').attr('type','text').attr('size','25').attr('aria-labelledby','move-message').addClass('form-control')
             .val(utils.url_path_join('/', that.notebook_path));
         var dialog_body = $('<div/>').append(
-            $("<p/>").addClass("rename-message")
+            $("<p/>").addClass("rename-message").attr('id', 'move-message')
                 .text(i18n.msg.sprintf(i18n.msg.ngettext("Enter a new destination directory path for this item:",
                 					"Enter a new destination directory path for these %d items:", num_items),num_items))
         ).append(
@@ -1135,12 +1295,14 @@ define([
     NotebookList.prototype.add_large_file_upload_button = function (file) {
         var that = this;
         var item = that.new_item(0, true);
+        var stop_signal = false;
         item.addClass('new-file');
         that.add_name_input(file.name, item, 'file');
         var cancel_button = $('<button/>').text("Cancel")
             .addClass("btn btn-default btn-xs")
             .click(function (e) {
                 item.remove();
+                stop_signal = true;
                 return false;
             });
 
@@ -1158,16 +1320,16 @@ define([
                     });
                     return false;
                 }
-                
+
                 var check_exist = function () {
                     var exists = false;
                     $.each(that.element.find('.list_item:not(.new-file)'), function(k,v){
                     if ($(v).data('name') === filename) { exists = true; return false; }
                     });
                     return exists
-                }
+                };
                 var exists = check_exist();
-                
+
                 var add_uploading_button = function (f, item) {
                     // change buttons, add a progress bar
                     var uploading_button = item.find('.upload_button').text("Uploading");
@@ -1183,14 +1345,17 @@ define([
                         .css('position', 'absolute');
 
                     var parse_large_file = function (f, item) {
-                        // codes inspired by http://stackoverflow.com/a/28318964
-                        var chunk_size = 1024 * 1024;
+                        // codes inspired by https://stackoverflow.com/a/28318964
+                        // 8MB chunk size chosen to match chunk sizes used by benchmark reference (AWS S3)
+                        var chunk_size = 1024 * 1024 * 8;
                         var offset = 0;
                         var chunk = 0;
                         var chunk_reader = null;
-                        var upload_file = null;
-                        
+
                         var large_reader_onload = function (event) {
+                            if (stop_signal === true) {
+                                return;
+                            }
                             if (event.target.error == null) {
                                 offset += chunk_size;
                                 if (offset >= f.size) {
@@ -1204,7 +1369,6 @@ define([
                                 upload_file(item, chunk);  // Do the upload
                             } else {
                                 console.log("Read error: " + event.target.error);
-                                return;
                             }
                         };
                         var on_error = function (event) {
@@ -1220,7 +1384,7 @@ define([
                                 body : "Failed to read file '" + name + "'",
                                 buttons : {'OK' : { 'class' : 'btn-primary' }}
                             });
-                        }
+                        };
 
                         chunk_reader = function (_offset, _f) {
                             var reader = new FileReader();
@@ -1234,18 +1398,25 @@ define([
                             reader.onerror = on_error;
                         };
 
+                        // This approach avoids triggering multiple GC pauses for large files.
+                        // Borrowed from kanaka's answer at:
+                        // https://stackoverflow.com/questions/12710001/how-to-convert-uint8-array-to-base64-encoded-string
+                        var  Uint8ToString = function(u8a){
+                            var CHUNK_SZ = 0x8000;
+                            var c = [];
+                            for (var i=0; i < u8a.length; i+=CHUNK_SZ) {
+                              c.push(String.fromCharCode.apply(null, u8a.subarray(i, i+CHUNK_SZ)));
+                            }
+                            return c.join("");
+                        };
+
                         // These codes to upload file in original class
                         var upload_file = function(item, chunk) {
                             var filedata = item.data('filedata');
                             if (filedata instanceof ArrayBuffer) {
                                 // base64-encode binary file data
-                                var bytes = '';
                                 var buf = new Uint8Array(filedata);
-                                var nbytes = buf.byteLength;
-                                for (var i=0; i<nbytes; i++) {
-                                    bytes += String.fromCharCode(buf[i]);
-                                }
-                                filedata = btoa(bytes);
+                                filedata = btoa(Uint8ToString(buf));
                                 format = 'base64';
                             }
                             var model = { name: filename, path: path };
@@ -1260,8 +1431,8 @@ define([
 
                             model.chunk = chunk;
                             model.content = filedata;
-                            
-                            var on_success = function (event) {
+
+                            var on_success = function () {
                                 if (offset < f.size) {
                                     // of to the next chunk
                                     chunk_reader(offset, f);
@@ -1278,7 +1449,7 @@ define([
                                 }
                             };
                             that.contents.save(path, model).then(on_success, on_error);
-                        }
+                        };
 
                         // now let's start the read with the first block
                         chunk_reader(offset, f);
@@ -1313,13 +1484,14 @@ define([
         item.find(".item_buttons").empty()
             .append(upload_button)
             .append(cancel_button);
-    }
+    };
 
     NotebookList.prototype.add_upload_button = function (item) {
         var that = this;
         var upload_button = $('<button/>').text(i18n.msg._("Upload"))
             .addClass('btn btn-primary btn-xs upload_button')
             .click(function (e) {
+                item.find('.upload_button').text("Uploading...");
                 var filename = item.find('.item_name > input').val();
                 var path = utils.url_path_join(that.notebook_path, filename);
                 var filedata = item.data('filedata');
@@ -1375,7 +1547,6 @@ define([
                     model.content = filedata;
                     content_type = 'application/octet-stream';
                 }
-                filedata = item.data('filedata');
 
                 var on_success = function () {
                     item.removeClass('new-file');
